/*
 *   main.c - Written by Sergey Kiselev
 *
 *   This file is part of spi_programmer.
 *
 *   spi_programmer is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   spi_programmer is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with spi_programmer.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Modified by Ján Sučan 
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <fcntl.h>
#include "parport_driver.h"
#include "sst25vf016b.h"

#define MEM_SIZE 0x200000	/* 2MB */
#define SEC_SIZE 4096LU
#define MEM_SECTORS (MEM_SIZE / SEC_SIZE)
#define ERASED_BYTE 0xff
#define MAX_TRIES 4

int str_to_sector_number(char *str, long *num);
void write_from_sector(unsigned long secnum, unsigned char *buf);
void verify_from_sector(unsigned long secnum, unsigned char *buf);
int erase_sector(unsigned long secnum);
void write_sector(unsigned long secnum, unsigned char *buf);
int verify_sector(unsigned long secnum, unsigned char *buf);

int
main(int argc, char *argv[])
{
        int fd;
        unsigned char *buffer;
        long sn;

        if (argc < 2) {
                fprintf (stderr, "Required argument missing\n");
                exit (1);
        }
        parport_init ();
        init ();
        if (!strcmp (argv[1], "id")) {
                printf ("Device ID: 0x%lx\n", Jedec_ID_Read());
        } else if (!strcmp (argv[1], "status")) {
                printf ("Status Register: 0x%x\n", Read_Status_Register());
        } else if (!strcmp (argv[1], "read")) {
                if (argc < 3) {
                        fprintf (stderr, "Filename missing\n");
                        exit (1);
                }
                if ((fd = creat (argv[2], 0666)) == -1) {
                        perror ("Unable to create file");
                        exit (2);
                }
                if ((buffer = malloc (MEM_SIZE)) == NULL) {
                        perror ("Unable to allocate memory");
                        exit (3);
                }
                HighSpeed_Read_Cont (0, MEM_SIZE, buffer);
                if (write (fd, buffer, MEM_SIZE) != MEM_SIZE) {
                        perror ("Error writing file");
                        exit (2);
                }
                close (fd);
        } else if (!strcmp (argv[1], "write")) {
                if (argc < 4) {
                        fprintf (stderr, "Sector number is missing\n");
                        exit (1);
                }
                if ((fd = open (argv[2], O_RDONLY)) == -1) {
                        perror ("Unable to open file");
                        exit (2);
                }
                if ((buffer = malloc (MEM_SIZE)) == NULL) {
                        perror ("Unable to allocate memory");
                        exit (3);
                }
                if (read (fd, buffer, MEM_SIZE) != MEM_SIZE) {
                        perror ("Error reading file");
                        exit (2);
                }
                close (fd);
                /* Start writing from the selected sector */
                if (str_to_sector_number(argv[3], &sn) != 0) {
			exit(1);
		}
		write_from_sector (sn, buffer);
		free (buffer);
		printf("OK: memory programmed\n");
	} else if (!strcmp (argv[1], "verify")) {
                if (argc < 4) {
                        fprintf (stderr, "Sector number is missing\n");
                        exit (1);
                }
                if ((fd = open (argv[2], O_RDONLY)) == -1) {
                        perror ("Unable to open file");
                        exit (2);
                }
                if ((buffer = malloc (MEM_SIZE)) == NULL) {
                        perror ("Unable to allocate memory");
                        exit (3);
                }
                if (read (fd, buffer, MEM_SIZE) != MEM_SIZE) {
                        perror ("Error reading file");
                        exit (2);
                }
                close (fd);
                /* Start verifying from the selected sector */
                if (str_to_sector_number(argv[3], &sn) != 0) {
			exit(1);
		}
		verify_from_sector (sn, buffer);
		free (buffer);
		printf("OK: memory verified\n");
        } else {
                fprintf(stderr, "Unknown command: %s\n", argv[1]);
                exit(1);
        }
        return 0;
}

int
str_to_sector_number(char *str, long *num) {
	char *c;

	*num = strtol(str, &c, 10);
	if ((str != '\0') && (*c == '\0')) {
		if ((*num >= 0) && (*num < MEM_SECTORS)) {
			return 0;
		} else { 
			fprintf(stderr, "ERROR: sector number %ld is outside range [0,%lu] \n",
				*num, MEM_SECTORS - 1);
		}
	} else {
		fprintf(stderr, "ERROR: not a number: %s\n", str);
	}
	return 1;
}

void
write_from_sector(unsigned long secnum, unsigned char *buf) {
        unsigned int ei, wi;

        for (; secnum < MEM_SECTORS; secnum++) {
                printf("Processing sector %lu\n", secnum);
                ei = wi = 0;
                while ((ei < MAX_TRIES) && (wi < MAX_TRIES)) {
                        /* Erase sector */
                        if (erase_sector(secnum) != 0) {
                                ei++;
                                continue;
                        } else {
                                ei = 0;
                        }
                        /* Write sector */
                        write_sector(secnum, buf);
                        /* Verify sector */
                        if (verify_sector(secnum, buf) != 0) {
                                wi++;
                                continue;
                        } else {
                                break;
                        }
                }
                /* Check if number of tries was exceeded */
                if (ei >= MAX_TRIES) {
                        printf("      ERROR: cannot erase sector\n");
                        exit(1);
                } else if (wi >= MAX_TRIES) {
                        printf("      ERROR: cannot write sector\n");
                        exit(1);
                }
        }
}

void
verify_from_sector(unsigned long secnum, unsigned char *buf) {
        unsigned int vi;

        for (; secnum < MEM_SECTORS; secnum++) {
                printf("Processing sector %lu\n", secnum);
                for (vi = 0; vi < MAX_TRIES; vi++) {
                        if (verify_sector(secnum, buf) == 0) {
                                break;
                        }
                }
                /* Check if number of tries was exceeded */
                if (vi >= MAX_TRIES) {
                        printf("      ERROR: sector verification failed\n");
                        exit(1);
                }
        }
}

int
erase_sector(unsigned long secnum)
{
        unsigned int i, j;
        unsigned char buffer[SEC_SIZE];
        unsigned char status_reg;

        status_reg = Read_Status_Register();
        status_reg &= 0x43;	/* clear BPL and BPx bits */
        EWSR();
        WRSR(status_reg);
        WREN();

        for (i = 0; i < MAX_TRIES; i++) {
                Sector_Erase(secnum * SEC_SIZE);
                usleep(2000);
                HighSpeed_Read_Cont(secnum * SEC_SIZE, SEC_SIZE, buffer);
                for (j = 0; j < SEC_SIZE; j++) {
                        if (buffer[j] != ERASED_BYTE)
                                break;
                }
                if (j < SEC_SIZE) {
                        continue;
                } else {
                        break;
                }
        }

        WRDI();

        return (i >= MAX_TRIES);
}

void
write_sector(unsigned long secnum, unsigned char *buf)
{
        unsigned long begin, address, end;
        unsigned long status_reg;	

        begin = secnum * SEC_SIZE;
        end = begin + SEC_SIZE;

        status_reg = Read_Status_Register();
        status_reg &= 0x43;	/* clear BPL and BPx bits */
        EWSR();
        WRSR(status_reg);
        WREN();
        EBSY();

        for (address = begin; address < end; address += 2) {
                if (address == begin)
                        Auto_Add_IncA_EBSY (address, buf[address], buf[address+1]);
                else
                        Auto_Add_IncB_EBSY (buf[address], buf[address+1]);
        }

        WRDI();
        DBSY();
}

int
verify_sector(unsigned long secnum, unsigned char *buf)
{
        unsigned int i, j;
        unsigned char sector[SEC_SIZE];

        for (i = 0; i < MAX_TRIES; i++) {
                HighSpeed_Read_Cont(secnum * SEC_SIZE, SEC_SIZE, sector);
                for (j = 0; j < SEC_SIZE; j++) {
                        if (sector[j] != buf[(secnum * SEC_SIZE) + j])
                                break;
                }
                if (j < SEC_SIZE) {
                        continue;
                } else {
                        break;
                }
        }

        return (i >= MAX_TRIES);
}
